from Orange.data import Table, Domain, StringVariable
from Orange.widgets import gui
from Orange.widgets.settings import Setting
from AnyQt.QtWidgets import QFileDialog
from Orange.widgets.widget import OWWidget, Input, Output, Msg
from Orange.data.pandas_compat import table_from_frame
import numpy as np
import logging
from itertools import cycle
import pandas as pd
import ploomber
from ploomber.executors import Serial
from ploomber.spec import DAGSpec
from ploomber import DAG
import os
import yaml
from ..base_widget import FilterWidget
from ..base_widget import BaseWidget, RC2Spectra
import tempfile
#from matplotlib.offsetbox import OffsetImage, AnnotationBbox

for handler in logging.root.handlers[:]:
    logging.root.removeHandler(handler)
logging.basicConfig(handlers=[logging.FileHandler("charisma_twinning.log", mode='w')], level=logging.NOTSET)
logging.root.setLevel(logging.NOTSET)
log = logging.getLogger("charisma")
log.info("log hijack for debugging")

class PloomberTwinningWidget(BaseWidget):
    # Define the widget's name, category, and outputs
    name = "CHARISMA Twinning protocol"
    description = """Protocol for twinning devices and spectra harmonization.\n\n
            - Input: 5 spectra per spectrometer using different laser power (mW).\n\n
            - Spectra treatment: Normalize Raman spectra (same power and integration time). Remove baseline.\n
            - LED correction: LED intensity correction.\n
            - Power regression line and Factor Correction (FC) calculation: Find 144 cm-1 peaks. Linear regression (laser power vs maximum intensity). FC = Slope A/ Slope B.\n
            - Harmonization: FC * spectra to harmonize\n
            - Quality calculation
            \n"""
    icon = "icons/twinning.png"
    priority = 10
    # want_main_area = False
    # resizing_enabled = False
    # proportion = Setting(50)
    commitOnChange = Setting(0)


    baseline_algorithm =  Setting("movmin") #als ,snip, none
    wlen =   Setting(10)    
    baseline_after_ledcorrection =  Setting(False)
    fit_peak = Setting(False)

    # label = Setting("")
    yaml_file = os.path.join(os.path.dirname(__file__), "ploomber","workflow_twinning", "pipeline_ploomber.yaml")
    env_file = os.path.join(os.path.dirname(__file__), "ploomber", "workflow_twinning","env_ploomber.yaml")


    should_auto_proc = Setting(False)

    class Inputs:
        reference_spe = Input("Raw data (RC2Spectra) of spectrometer A (REFERENCE device)", RC2Spectra, default=True)
        twinned_spe = Input("Raw data (RC2Spectra) of spectrometer B (device to TWIN)", RC2Spectra, default=False)
        reference_led = Input("Reference device LED spectra (RC2Spectra)", RC2Spectra, default=False)
        twinned_led = Input("Twinned device LED spectra (RC2Spectra)", RC2Spectra, default=False)      
        data_env = Input("Settings", Table, default=False)   

    class Outputs:
#        reference_spe = Output("Reference (RC2Spectra)", RC2Spectra, default=True)
#        twinned_spe = Output("Twinned (RC2Spectra)", RC2Spectra, default=True)
        out_spe = Output("RC2Spectra", RC2Spectra, default=True)
        data = Output("Data", Table, default=False)
        meta_spectra = Output("Metadata spectra", Table, default=False)
        meta_leds = Output("Metadata LEDs", Table, default=False)
               

    @Inputs.reference_spe
    def set_reference_spe(self, spe):
        if spe:
            self.reference_spe = spe
        else:
            self.reference_spe = None
        self.update_inputs_info()


    @Inputs.twinned_spe
    def set_twinned_spe(self, spe):
        if spe:
            self.twinned_spe = spe
        else:
            self.twinned_spe = None
        self.update_inputs_info()
    

    @Inputs.reference_led
    def set_reference_led(self, spe):
        if spe:
            self.reference_led = spe
        else:
            self.reference_led = None
        self.update_inputs_info()

    @Inputs.twinned_led
    def set_twinned_led(self, spe):
        if spe:
            self.twinned_led = spe
        else:
            self.twinned_led = None
        self.update_inputs_info()
        

    @Inputs.data_env
    def set_data_env(self, data_env):
        self.Error.processing_error.clear()
        if data_env:
            self.data_env = data_env
        else:
            self.data_env = None

    def update_inputs_info(self):
        self.Warning.clear()
        len11 = len(self.reference_spe) if self.reference_spe else None
        len12 = len(self.reference_led) if self.reference_led else None
        len21 = len(self.twinned_spe) if self.twinned_spe else None
        len22 = len(self.twinned_led) if self.twinned_led else None        
        self.info.set_input_summary(f' (Reference {len11} RC2Spectra + {len12} LED RC2Spectra)\n (Twinned {len21} RC2Spectra + {len22} LED RC2Spectra)')
        
    # same class can be initiated for Error and Information messages
    class Warning(OWWidget.Warning):
        warning = Msg("Warning")

    class Information(OWWidget.Information):
        success = Msg("Workflow successful")

    class Error(OWWidget.Error):
        processing_error = Msg("Workflow error")

    def env2table(self):
        with open(self.env_file, "r") as file:
            env = yaml.safe_load(file)        
        self.env = env
        self.env["output_folder"] = os.path.join(tempfile.gettempdir(),"ploomber_twinning")
        self.set_data_env(table_from_frame(pd.DataFrame.from_dict(env, orient="index", columns=["value"])))

    def __init__(self):
        # Initialize the widget
        super().__init__()

        # Load the environment dictionary from the env.yaml file
        self.env2table()
        self.out_spe = RC2Spectra()
        self.dag=None
        box = gui.widgetBox(self.controlArea, self.name)
        #gui.button(box, self, "Select ENV File", callback=self.load_file_env)
        gui.button(box, self, "Load pipeline", callback=self.load_workflow)
        gui.button(box, self, "Prepare input", callback=self.prepare_input)
        #gui.button(box, self, "Run", callback=self.run_workflow)
        #gui.button(self.optionsBox, self, "Commit", callback=self.commit)
        #self.optionsBox.setDisabled(False)
        cbox = gui.widgetBox(box, "Baseline removal")
        gui.spin(box, self, 'wlen', 1, 5000, step=20, callback=self.auto_process, label='Window length')     
        cbox = gui.widgetBox(box, "Peak fitting")   

    def load_file_env(self):
        filenames, filt = QFileDialog.getOpenFileNames(
            caption='Select ENV File',
            directory='',
            filter='YAML Files (*.yaml *.yml);;All Files (*)',
            initialFilter='All files (*)',
            )
        if filenames:
            self.env_file = filenames[0]
            self.env2table()   

    def on_finish(self,dag,report):
        #print(dag)
        log.info(report)

    def on_render(self,dag):
        log.info("render ",dag)

    def on_failure(self,dag,report):
        print(report)

    def load_workflow(self):
        if not self.yaml_file or not self.env_file:
            log.info("Please select both YAML and environment files.")
            return
        try:
            log.info(self.yaml_file,self.env_file,self.env)
            self.dag_spec = DAGSpec(data= self.yaml_file, env = self.env)
            self.dag = self.dag_spec.to_dag()
            self.dag.on_finish = self.on_finish
            self.dag.on_render = self.on_render

            log.info(self.dag.status())
            self.plot_spe()
            log.info("Workflow loaded successfully.")
        except Exception as e:
            log.info(e)
            self.Error.processing_error(f"Error: {str(e)}")

    def workflow_input_file(self):
        return os.path.join( self.env["output_folder"],self.env["input_rcspectra"])

    def prepare_input(self):
        df_spectra = None
        df_leds = None
        log.info("prepare_input")
        input_rcspectra = self.workflow_input_file()
        #log.info("prepare_input  {}".format(input_rcspectra))               
        try:
            A = RC2Spectra()
            for spe in self.reference_spe:
                A.append(spe)
            self.send_outputs()
            log.info(len(A.data))
            B = RC2Spectra()
            for spe in self.twinned_spe:
                B.append(spe)
            dfA= pd.DataFrame(A.data,columns=["spectrum"])
            
            dfA["reference"]= True
            dfB= pd.DataFrame(B.data,columns=["spectrum"])
            dfB["reference"]= False
            df_spectra = pd.concat([dfA,dfB], ignore_index=True)  
            df_spectra.to_hdf(input_rcspectra, key=self.env['key_spectra'], mode='w')                  
        except Exception as err:
            log.error(err)

        try:
            A = RC2Spectra()
            for spe in self.reference_led:
                A.append(spe)
            B = RC2Spectra()
            for spe in self.twinned_led:
                B.append(spe)
            log.info(len(A.data))
            dfA= pd.DataFrame(A.data,columns=["spectrum"])
            dfA["reference"]= True
            dfB= pd.DataFrame(B.data,columns=["spectrum"])
            dfB["reference"]= False
            df_leds = pd.concat([dfA,dfB], ignore_index=True)    
            df_leds.to_hdf(input_rcspectra, key=self.env['key_leds'], mode='a')
        except Exception as err:
            log.error(err)
        self.set_meta_spectra(table_from_frame(df_spectra))
        self.set_meta_leds(table_from_frame(df_leds))
      # reference_spe = Input("Raw data (RC2Spectra) of spectrometer A (REFERENCE device)", RC2Spectra, default=True)
      #  twinned_spe = Input("Raw data (RC2Spectra) of spectrometer B (device to TWIN)", RC2Spectra, default=False)
      #  reference_led = Input("Reference device LED spectra (RC2Spectra)", RC2Spectra, default=True)
      #  twinned_led = Input("Twinned device LED spectra (RC2Spectra)", RC2Spectra, default=False)      
      #  data_env = Input("Settings", Table, default=False)               

    def run_workflow(self):
        if self.dag is None:
            self.load_workflow()
        
        try:
            log.info("prepare input")
            self.prepare_input()

        except Exception as err:
            log.error(err)

        try:
            self.dag.build(force=False)
            devices_h5file = os.path.join( self.env["output_folder"],"twinning_spectra_table_harmonized.h5")
            results = pd.read_hdf(devices_h5file, key='devices')
            #devices_h5file= upstream["twinning_peaks"]["data"]
            #devices = pd.read_hdf(devices_h5file, "devices")
            #devices.head()
            #processing = pd.read_hdf(devices_h5file, "processing")

            for index,row in results.iterrows():
                if row["reference"]:
                    key="spectrum_corrected"
                else:
                    key="spectrum_harmonized"
                try:    
                    spe = row[key]
                    sc1 = spe.trim_axes(method='x-axis', boundaries=(100,max(spe.x)-1000))
                    meta={}
                    for tag in ["laser_power","reference","device","laser_power_percent"]:
                        meta[tag] = row[tag]
                    self.out_spe.append(sc1)
                    #rc2.spectrum.Spectrum(sc1.x,sc1.y,meta))
                except Exception as err:
                    print(err)
                #,label="{}".format(row["device"]))
            self.send_outputs()
            self.Information.success()
            #self.statusBar().showMessage("Workflow executed successfully.")
        except Exception as e:
            log.error(e)
            self.Error.processing_error(f"Error: {str(e)}")
            #self.statusBar().showMessage(f"Error: {str(e)}")            

    def set_meta_spectra(self, data):
        self.Error.processing_error.clear()
        if data:
            self.meta_spectra = data
        else:
            self.meta_spectra = None

    def set_meta_leds(self, data):
        self.Error.processing_error.clear()
        if data:
            self.meta_leds = data
        else:
            self.meta_leds = None
    
    def send_report(self):
        pass


    def process(self):
        self.run_workflow()

    def custom_plot(self, ax):      
        if self.dag is None:
            pass
        else:
            pass
            #ipython_image  = self.dag.plot("embed")
            #imagebox = OffsetImage(ipython_image, zoom=0.5)  # You can adjust the zoom factor as needed
            #ab = AnnotationBbox(imagebox, (0.5, 0.5), frameon=False, xycoords='axes fraction')
            #ax.add_artist(ab)

            # Optional: Customize the plot or axis as needed
            #ax.set_xlim(0, 1)
            #ax.set_ylim(0, 1)
            #ax.set_aspect('equal')            

if __name__ == "__main__":
    from Orange.widgets.utils.widgetpreview import WidgetPreview
    from Orange.data import Table
    import os
    try:
        WidgetPreview(PloomberTwinningWidget).run()
    except Exception as err:
        log.error(err)
        WidgetPreview(PloomberTwinningWidget).run()



# + tags=["parameters"]
upstream = ["twinning_peaks"]
product = None
probe: None

# -

import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
import numpy as np


def plot_spectra(row, axes,column=0, reference=True,match_led= None,leds = None, cmap=None, norm = None, fc= None):
    peak= 144
    _left = peak - 50
    _right = peak + 50
    _right_long = 1750
    _color = cmap(norm(row["laser_power_percent"]))
    #_baseline_when = row["baseline_removed"]
    try:
        sc=row["spectrum"]
        sc.plot(ax=axes[0][column],label="{}%".format(row["laser_power_percent"]),c=_color)
        sc = sc.trim_axes(method='x-axis', boundaries=(_left, _right))        
        sc.plot(ax=axes[1][column],label="{}%".format(row["laser_power_percent"]),c=_color)
    except:
        pass


    index=2
    step = 1
    for _tag in ["spectrum_normalized","spectrum_normalized_baseline","spectrum_corrected","spectrum_corrected_baseline","spectrum_harmonized"]:
        try:
            #if _baseline_when=="before LED correction":
            sc=row[_tag]

            _fc = ""
            boundaries =  [(_left, _right)]
            if (_tag == "spectrum_harmonized") :
                _fc = "FC {:.3e}".format(fc)
                boundaries = [(_left, _right),(_left, _right_long)]
            if (_tag in ["spectrum_corrected_baseline"]) :
                boundaries = [(_left, _right),(_left, _right_long)]      
            if (_tag in ["spectrum_corrected"]) & reference :
                boundaries = [(_left, _right),(_left, _right_long)]                             
            for b in boundaries:
                sc1 = sc.trim_axes(method='x-axis', boundaries=b)
                sc1.plot(ax=axes[index][column],label="{}%".format( row["laser_power_percent"]),c=_color)
                axes[index][column].set_title("{}. {} {}".format(step,_tag.replace("_"," "),_fc))
                index =index+1  
            step = step+ 1          
        except Exception as err:
            print(err)
            pass    
       

    try:
        device=row["device"]
       
        led = match_led.loc[device]["led_spectra"]
        sc = leds.loc[led]["spectrum"]
        sc.plot(ax=axes[0][column],label='_nolegend_',c="gray")
        sc = sc.trim_axes(method='x-axis', boundaries=(_left, _right))        
        sc.plot(ax=axes[1][column],label='_nolegend_',c="gray")
        sc.plot(ax=axes[2][column],label='_nolegend_',c="gray")
    except Exception as err:
        print(err)
        pass   


    axes[0][column].set_title("0. {} {} {}".format(row["device"],row["probe"],"(reference)" if reference else ""))
    axes[1][column].set_title("0. (cropped)")

    #plt.legend(loc='upper left', bbox_to_anchor=(1, 1))

devices_h5file= upstream["twinning_peaks"]["data"]
devices = pd.read_hdf(devices_h5file, "devices")
devices.head()

led_frame = pd.read_hdf(devices_h5file, "led")
led_frame.head()

processing = pd.read_hdf(devices_h5file, "processing")
processing.head()

regression = pd.read_hdf(devices_h5file, "regression")
factor_correction = pd.read_hdf(devices_h5file, "factor_correction")
regression,factor_correction

print(factor_correction.iloc[0,0])

leds = pd.read_hdf(devices_h5file, "led")
leds.head()

#match_led = pd.read_hdf(devices_h5file, "match")
#match_led.head()
match_led={
    "reference" : "reference",
    "twinning": "twinning"
}
match_led
    
print(processing.index,processing["field"])
 
cmap = plt.get_cmap('plasma')
norm = mcolors.Normalize(vmin=0, vmax=100)
reference_condition = (devices["reference"]) & (devices["probe"] == probe)
A=devices.loc[reference_condition].sort_values(by='laser_power_percent')
twinned_condition = (~devices["reference"]) & (devices["probe"] == probe)
B =devices.loc[twinned_condition].sort_values(by='laser_power_percent')

A

B

_figlen = 7 
print(processing.shape[0])
fig, axes = plt.subplots(_figlen,2 , figsize=(14,14)) 
A.apply(lambda row: plot_spectra(row,axes, 0, True,match_led,leds,cmap,norm), axis=1)
B.apply(lambda row: plot_spectra(row,axes, 1,False,match_led,leds,cmap,norm,fc=factor_correction.iloc[0,0]), axis=1)
plt.tight_layout()


fig, axes = plt.subplots(1,2, figsize=(10,4)) 
axes[0].plot(A["laser_power"],A["height"],'o',label=A["device"].unique())
A_regr=regression.loc["reference"]
A_pred = A["laser_power"]*A_regr["slope"] + A_regr["intercept"]
axes[0].plot(A["laser_power"],A_pred,'-',label="{:.2e} * LP + {:.2e}".format(A_regr["slope"],A_regr["intercept"]))
B_regr=regression.loc["twinned"]
axes[0].plot(B["laser_power"],B["height"],'+',label=B["device"].unique())
B_pred =B["laser_power"]*B_regr["slope"] + B_regr["intercept"]
axes[0].plot(B["laser_power"],B_pred,'-',label="{:.2e} * LP + {:.2e}".format(B_regr["slope"],B_regr["intercept"]))
#plt.plot(x_values, y_values, color='red', label="Linear Regression Line")
axes[0].set_ylabel("height of the (fitted) peak @ 144cm-1")
axes[0].set_xlabel("laser power, %")
axes[0].legend()
bar_width = 0.2  # Adjust this value to control the width of the groups
bar_positions = np.arange(len(A["laser_power_percent"].values))
axes[1].bar(bar_positions -bar_width,A["area"], width=bar_width,label=str(A["device"].unique()))
bar_positions = np.arange(len(B["laser_power_percent"].values))
axes[1].bar(bar_positions  ,B["area"],width=bar_width,label=str(B["device"].unique()))
axes[1].bar(bar_positions + bar_width,B["area_harmonized"],width=bar_width,label="{} harmonized FC={:.2e}".format(B["device"].unique(),factor_correction.iloc[0,0]))
axes[1].set_ylabel("spectrum area")
axes[1].set_xlabel("laser power, %")
# Set the x-axis positions and labels
plt.xticks(bar_positions, B["laser_power_percent"])
axes[1].legend()
plt.tight_layout()


show_leds = False
if show_leds:
    for index, led_spectra in leds.iterrows():
        fig, axes = plt.subplots(1, 3, figsize=(15, 2))   
        spe_led = led_spectra["spectrum"]   
        spe_led.plot(label=index,ax=axes[0])
        area = led_spectra["area"]
        spe_dist = led_spectra["spe_dist"]
        axes[1].plot(spe_led.x,spe_dist.pdf(spe_led.x))
        axes[2].plot(spe_led.x,spe_dist.pdf(spe_led.x)*area)


from .base_widget import BaseWidget
from .types import RC2Spectra


class CreatorWidget(BaseWidget, openclass=True):
    def __init__(self):
        super().__init__()
        self.in_spe = RC2Spectra()



from Orange.widgets.widget import Input
from ..base_widget import BaseWidget, RC2Spectra


class Merger(BaseWidget):
    name = "Merger"
    description = "merge spectra"
    icon = "icons/spectra.svg"

    def __init__(self):
        self.in_spe_dict = dict()
        super().__init__()

    def input_hook(self):
        while self.select_box.takeItem(0):
            pass
        for spe_i, spe in enumerate(self.in_spe):
            self.select_box.addItem(f'{spe_i}: {spe!r}')

    class Inputs:
        in_spe = Input("RC2Spectra", RC2Spectra, multiple=True, auto_summary = False)

    @Inputs.in_spe
    def set_in_spe(self, spe, id_):
        self.should_auto_plot = False
        if id_ in self.in_spe_dict:
            if spe is None:
                del self.in_spe_dict[id_]
            else:
                self.in_spe_dict[id_] = spe
        else:
            self.in_spe_dict[id_] = spe

        if self.in_spe_dict:
            self.auto_process()

    def process(self):
        self.out_spe = list()
        for id_, spes in self.in_spe_dict.items():
            self.out_spe.extend(spes)
        self.send_outputs()



# + tags=["parameters"]
upstream = None
product = None
input_folder = None
hsds_investigation = None
hsds_instrument = None
hsds_provider = None
# -

import os
import glob
import re
from fuzzywuzzy import fuzz
import pandas as pd
import ramanchada2 as rc2

tags = { '785nm':"wavelength", 
        '532nm':"wavelength", 
        '785':"wavelength", 
        '532':"wavelength", 
        '150mW':"laser_power",
        '327mW':"laser_power",
        '(200mW)':"laser_power",      
        '20%':"laser_power_percent",   
          '150ms' : "acquisition_time",
          '150msx5ac' : "acquisition_time",
          'Probe': "probe", 
          '1': "replicate",
          '2': "replicate",
          '3': "replicate",
          '4': "replicate",
         '5': "replicate",
         'position 1': "replicate",
         'txt' : "extension",
         'Neon' : 'sample',
         'Ti' : 'sample',
         'nCal' : 'sample',
         'sCal' : 'sample',
         'Si' : 'sample',
         'S1' : 'sample',
         'S0B' : 'sample',
         'PST' : 'sample',
         'LED' : 'sample',
         'NIR' : 'sample',
         'OP01' : "optical_path"
         }

_lookup = {
    "Original file" : "Original file",
    "laser_wavelength" : "wavelength",
    "model" : "instrument",
    "title" : "title",
    "device" : "instrument",
    "laser_wavelength" : "wavelength",
    "laser_powerlevel" : "laser_power_percent",
    "intigration times(ms)":"acquisition_time",
    "integration times(ms)":"acquisition_time",
    "integ_time" :  "acquisition_time"      
}
def fuzzy_match(vals,tags):
    parsed = {}
    parsed_similarity= {}
    for val in vals:
        similarity_score = None
        match = None
        for tag in tags:
            _tmp = fuzz.ratio(val,tag)
            if (similarity_score is None) or (similarity_score < _tmp):
                similarity_score =_tmp
                match = tags[tag]
        if not match in parsed:
            parsed[match]  = val
            parsed_similarity[match] = similarity_score
        else:
            if parsed_similarity[match] < similarity_score:
                parsed[match]  = val
                parsed_similarity[match] = similarity_score        
    if "wavelength" in parsed:
         parsed["wavelength"] = re.findall(r'\d+', parsed["wavelength"])[0]
    if "extension" in parsed:
        del parsed["extension"]            
    return (parsed,parsed_similarity)

def list_files_in_directory(directory, file_pattern='*.*'):
    metadata = {}
    files = glob.glob(os.path.join(directory, '**', file_pattern), recursive=True)
    for file in files:
        try:
            spe=rc2.spectrum.from_local_file(file)
            #print(spe.meta)
            file_name = os.path.basename(file)
            file_name, file_extension = os.path.splitext(file_name)
            basename = re.split(r'[-_()]+', file_name)
            if file_extension in ["json","xlsx"]:
                continue
            parsed,parsed_similarity = fuzzy_match([s for s in basename if s],tags)
            for m in ["Original file","laser_wavelength","wl","model","title","laser_powerlevel","integration times(ms)","intigration times(ms)","integ_time","device","spectrometer"]:
                try:
                    parsed[_lookup[m]] = spe.meta[m]
                except:
                    pass
            relative_path = os.path.relpath(file, directory)
            metadata[relative_path] = parsed
        except:
            pass
    return metadata



metadata = list_files_in_directory(input_folder)

df = pd.DataFrame.from_dict(metadata, orient="index")
df["hsds_investigation"] = hsds_investigation
df["hsds_provider"] = hsds_provider
df["hsds_instrument"] = hsds_instrument
df["enabled"] = True
df["delete"] = False

with pd.ExcelWriter(product["data"], engine='openpyxl') as writer:
    df.to_excel(writer,sheet_name="files")
    pd.DataFrame({"value": [input_folder]}, index=["path"]).to_excel(writer, sheet_name='paths')


from Orange.widgets import gui
from Orange.widgets.settings import Setting
from ramanchada2.misc.types.peak_candidates import ListPeakCandidateMultiModel

from ..base_widget import FilterWidget


class FindPeaks(FilterWidget):
    name = "Find Peaks"
    description = "Find Peaks"
    icon = "icons/spectra.svg"

    prominence_val = Setting(5)
    hht_chain_str = Setting('80 20')
    is_sharpening_enabled = Setting(True)
    wlen = Setting(200)
    min_peak_width = Setting(2)

    def __init__(self):
        super().__init__()
        box = gui.widgetBox(self.controlArea, self.name)
        self.hht_chain_edit = gui.lineEdit(box, self, 'hht_chain_str', label='HHT Chain', callback=self.auto_process)
        gui.checkBox(box, self, "is_sharpening_enabled", "Enable sharpening", callback=self.auto_process,
                     stateWhenDisabled=False, disables=self.hht_chain_edit)
        self.is_sharpening_enabled = False

        self.prominence_val_spin = gui.doubleSpin(box, self, 'prominence_val', 0, 1000, decimals=2,
                                                  label='Prominence [×σ]', step=.5, callback=self.auto_process)
        gui.spin(box, self, 'wlen', 1, 5000, step=20, callback=self.auto_process, label='Window length')
        gui.spin(box, self, 'min_peak_width', 1, 5000, callback=self.auto_process, label='Min peak width')

    def process(self):
        hht_chain = [int(i) for i in self.hht_chain_str.split()]
        self.hht_chain_str = ' '.join([str(i) for i in hht_chain])
        self.out_spe = list()
        for spe in self.in_spe:
            prominence = spe.y_noise * self.prominence_val
            self.out_spe.append(
                spe.find_peak_multipeak_filter(
                    wlen=self.wlen,
                    width=self.min_peak_width,
                    hht_chain=hht_chain,
                    sharpening=('hht' if self.is_sharpening_enabled else None),
                    prominence=prominence)
                )
        self.send_outputs()

    def custom_plot(self, ax):
        for spe in self.out_spe:
            ListPeakCandidateMultiModel.validate(spe.result).plot(ax)



#!/usr/bin/env python

from os import path, walk

from setuptools import find_namespace_packages, setup

NAME = 'oranchada'

VERSION = '1.0.0'

DESCRIPTION = 'Orange add-on for Raman spectroscopy'
README_FILE = path.join(path.dirname(__file__), 'README.pypi')
LONG_DESCRIPTION = open(README_FILE, 'r', encoding='utf-8').read()
LONG_DESCRIPTION_CONTENT_TYPE = 'text/markdown'
URL = 'https://github.com/h2020charisma/oranchada'
AUTHOR = 'IDEAconsult Ltd.'
AUTHOR_EMAIL = 'dev-charisma@ideaconsult.net'
LICENSE = 'MIT'

KEYWORDS = [
    # [PyPi](https://pypi.python.org) packages with keyword "orange3 add-on"
    # can be installed using the Orange Add-on Manager
    'Raman',
    'oranchada',
    'orange3 add-on',
    'spectroscopy',
]

PYTHON_REQUIRES = '>3.8.0'

PACKAGE_DATA = {}

PACKAGE_DIR = { '': 'src' }

PACKAGES = find_namespace_packages(where='src', exclude=['tests'])

DATA_FILES = [
    # Data files that will be installed outside site-packages folder
]

INSTALL_REQUIRES = [
    'Orange3>=3.31.0',
    'fuzzywuzzy~=0.18.0',
    'numpy>=1.18.0',
    'orange-canvas-core>=0.1.24',
    'orange-widget-base>=4.16.1',
    'pip>=9.0',  # same as for Orange 3.28
    'ploomber~=0.23.0',
    'ramanchada2~=1.0',
    'setuptools>=36.3',  # same as for Orange 3.28
    'tables~=3.9.1',
    # 'AnyQt>=0.0.6',
    # 'bottleneck',
    # 'colorcet',
    # 'extranormal3 >=0.0.3',
    # 'h5py',
    # 'lmfit>=1.0.2',
    # 'pebble',
    # 'pillow',
    # 'pyqtgraph>=0.11.1,!=0.12.4',  # https://github.com/pyqtgraph/pyqtgraph/issues/2237
    # 'renishawWiRE>=0.1.8',
    # 'scikit-learn>0.23.0',
    # 'scipy>=1.4.0',
    # 'serverfiles>=0.2',
    # 'spectral>=0.18,!=0.23',
]

EXTRAS_REQUIRE = {
    'test': ['coverage'],
    # 'test': ['pytest', 'coverage'],
    # 'doc': ['sphinx', 'recommonmark', 'sphinx_rtd_theme'],
}

ENTRY_POINTS = {
    # Entry points that marks this package as an orange add-on. If set, addon will
    # be shown in the add-ons manager even if not published on PyPi.
    'orange3.addon': (
        'oranchada = orangecontrib.oranchada',
    ),

    # Entry point used to specify packages containing tutorials accessible
    # from welcome screen. Tutorials are saved Orange Workflows (.ows files).
    'orange.widgets.tutorials': (
        # Syntax: any_text = path.to.package.containing.tutorials
        'infraredtutorials = orangecontrib.oranchada.tutorials',
    ),

    # Entry point used to specify packages containing widgets.
    'orange.widgets': (
        # Syntax: category name = path.to.package.containing.widgets
        # Widget category specification can be seen in
        #    orangecontrib/example/widgets/__init__.py
        'Oranchada Easy = orangecontrib.oranchada.widgets_easy',
        'Oranchada Pro = orangecontrib.oranchada.widgets_pro',
    ),

    # Register widget help
    "orange.canvas.help": (
        'html-index = orangecontrib.oranchada.widgets:WIDGET_HELP_PATH',)

}

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Plugins',
    'Intended Audience :: Developers',
    'Intended Audience :: Education',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3 :: Only',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python',
    'Topic :: Scientific/Engineering :: Artificial Intelligence',
    'Topic :: Scientific/Engineering :: Bio-Informatics',
    'Topic :: Scientific/Engineering :: Visualization',
    'Topic :: Scientific/Engineering',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Software Development :: Libraries',
]

TEST_SUITE = "orangecontrib.oranchada.tests.suite"


def include_documentation(local_dir, install_dir):
    global DATA_FILES

    doc_files = []
    for dirpath, _, files in walk(local_dir):
        doc_files.append((dirpath.replace(local_dir, install_dir),
                          [path.join(dirpath, f) for f in files]))
    DATA_FILES.extend(doc_files)


def setup_package():
    # cmdclass = {
    #     'coverage': CoverageCommand,
    #     'lint': LintCommand,
    # }

    # include_documentation('doc/build/htmlhelp', 'help/orange-spectroscopy')

    setup(
        author=AUTHOR,
        author_email=AUTHOR_EMAIL,
        classifiers=CLASSIFIERS,
        data_files=DATA_FILES,
        description=DESCRIPTION,
        entry_points=ENTRY_POINTS,
        extras_require=EXTRAS_REQUIRE,
        include_package_data=True,
        install_requires=INSTALL_REQUIRES,
        keywords=KEYWORDS,
        license=LICENSE,
        long_description=LONG_DESCRIPTION,
        long_description_content_type=LONG_DESCRIPTION_CONTENT_TYPE,
        name=NAME,
        package_dir=PACKAGE_DIR,
        package_data=PACKAGE_DATA,
        packages=PACKAGES,
        python_requires=PYTHON_REQUIRES,
        url=URL,
        version=VERSION,
        zip_safe=False,
    )


if __name__ == '__main__':
    setup_package()



from Orange.widgets.widget import Input, Msg

from ..base_widget import ArithmeticWidget
from ..base_widget.types import RC2Spectra


class Subtract(ArithmeticWidget):
    name = "Subtract"
    description = "subtract spectra\nDifference spectra = Minuends - Subtrahends"
    icon = "icons/spectra.svg"

    class Inputs:
        minuend = Input("Minuend (RC2Spectra)", RC2Spectra, auto_summary = False)
        subtrahend = Input("Subtrahend (RC2Spectra)", RC2Spectra, auto_summary = False)

    class Warning(ArithmeticWidget.Warning):
        operands_dont_broadcast = Msg('Operands do not broadcast')

    def update_inputs_info(self):
        self.Warning.clear()
        len_minuend = len(self.minuend) if self.minuend else None
        len_subtrahend = len(self.subtrahend) if self.subtrahend else None
        self.info.set_input_summary(f'{len_minuend} RC2Spectra - {len_subtrahend} RC2Spectra')

    @Inputs.minuend
    def set_minuend(self, spe):
        if spe:
            self.minuend = spe
        else:
            self.minuend = None
        self.update_inputs_info()
        self.auto_process()

    @Inputs.subtrahend
    def set_subtrahend(self, spe):
        if spe:
            self.subtrahend = spe
        else:
            self.subtrahend = None
        self.update_inputs_info()
        self.auto_process()

    def __init__(self):
        super().__init__()
        self.minuend = None
        self.subtrahend = None
        self.info.set_input_summary(self.info.NoInput)

    def process(self):
        self.out_spe = list()
        if self.minuend and self.subtrahend:
            len1 = len(self.minuend)
            len2 = len(self.subtrahend)
            if len1 == len2:
                for i in range(len1):
                    self.out_spe.append(self.minuend[i] - self.subtrahend[i])
            elif len1 == 1:
                for i in range(len2):
                    self.out_spe.append(self.minuend[0] - self.subtrahend[i])
            elif len2 == 1:
                for i in range(len1):
                    self.out_spe.append(self.minuend[i] - self.subtrahend[0])
            else:
                self.Warning.operands_dont_broadcast()
        self.send_outputs()



import numpy as np
from Orange.widgets import gui
from Orange.widgets.settings import Setting

from Orange.widgets.widget import  Output
from Orange.data import Table

from ..base_widget import CreatorWidget
from ramanchada2.protocols.calibration import  YCalibrationCertificate
from ramanchada2.spectrum import Spectrum

from Orange.data.pandas_compat import table_from_frame
import pandas as pd

class EditYCalibrationCertificateWidget(CreatorWidget):
    name = "Edit Y Calibration Certificate"
    description = "Edit Y Calibration Certificate and return results as a table"
    icon = "icons/spectra.svg"

    id = Setting("NIR785_EL0-9002")
    wavelength = Setting(785)
    params = Setting("A0 = 5.90134423e-1,A = 5.52032185e-1,B = 5.72123096e-7,x0 = 2.65628776e+3")
    equation = Setting("A0 + A * np.exp(-B * (x - x0)**2)")
    raman_shift_min = Setting(200)
    raman_shift_max = Setting(3500)
    temperature_c_min = Setting(20)
    temperature_c_max = Setting(25)    
    
    class Outputs(CreatorWidget.Outputs):
        out_table = Output("Edited Certificate Table", Table)



    def __init__(self):
        super().__init__()

        mbox = gui.widgetBox(self.controlArea, "Mandatory")
        gui.doubleSpin(mbox, self, 'wavelength', 0, 5000, decimals=0, step=1, 
                       callback=self.auto_process,
                       label='Laser Wavelength [nm]')   
        gui.lineEdit(mbox, self, "id", label="ID:")
        #gui.lineEdit(box, self, "description", label="Description:")
        #gui.lineEdit(mbox, self, "url", label="URL:")
        gui.lineEdit(mbox, self, "params", label="Params:",callback=self.auto_process)
        gui.lineEdit(mbox, self, "equation", label="Equation:",callback=self.auto_process)
     
        obox = gui.widgetBox(self.controlArea, "Optional")
        gui.spin(obox, self, "raman_shift_min", 0, 200, label="Raman Shift Min:",callback=self.auto_process)
        gui.spin(obox, self, "raman_shift_max", 50, 4500, label="Raman Shift Max:",callback=self.auto_process)     
        gui.spin(obox, self, "temperature_c_min", -100, 100, label="Temperature Min (C):",callback=self.auto_process)
        gui.spin(obox, self, "temperature_c_max", -100, 100, label="Temperature Max (C):",callback=self.auto_process)
        self.auto_process()
    
    def process(self):
        _cert = YCalibrationCertificate(
            id=self.id,
            description="",
            url="",
            wavelength=self.wavelength,
            params=self.params,
            equation=self.equation,
            temperature_c=(self.temperature_c_min, self.temperature_c_max),
            raman_shift=(self.raman_shift_min, self.raman_shift_max)
        )        
        if _cert.raman_shift is None:
            x = np.linspace(100, 4000)
        else:
            x = np.linspace(_cert.raman_shift[0], _cert.raman_shift[1])
        y = _cert.Y(x)
        self.out_spe = [Spectrum(x=x,y=y)]

        self.send_outputs()

# Create an Orange data table from the certificate
        columns = [
            "ID",
            "Wavelength",
            "Params",
            "Equation",
            "Temperature Min",
            "Temperature Max",
            "Raman Shift Min",
            "Raman Shift Max"
        ]

        data = [[
            _cert.id,
            int(_cert.wavelength),
            _cert.params,
            _cert.equation,
            _cert.temperature_c[0],
            _cert.temperature_c[1],
            _cert.raman_shift[0],
            _cert.raman_shift[1]
        ]]
        df = pd.DataFrame(data, columns=columns)
        self.Outputs.out_table.send(table_from_frame(df))        